#!/bin/bash
#
#  Copyright 2010 Rackspace US, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

export ROOT="${ROOT:-}"
export AGENTSMITHDIR="${AGENTSMITHDIR:-/usr/share/agent-smith}"
export TEMP="${TEMP:-$ROOT/tmp}"

. "${AGENTSMITHDIR}/common.sh"
. "${AGENTSMITHDIR}/network/redhat.sh"
. "${AGENTSMITHDIR}/network/gentoo.sh"
. "${AGENTSMITHDIR}/network/debian.sh"
. "${AGENTSMITHDIR}/network/arch.sh"

function get_interface_bullshit() {
    set -a public_addresses
    set -a private_addresses
    set -a dns
    set -a routes

    # This assumes that /etc/hostname is already configured
    if [ -e $ROOT/etc/hostname ]; then 
        hostname=$( cat $ROOT/etc/hostname )
    fi

    while read network_configuration; do
        set -- `echo $network_configuration | sed -e 's/:/ /' -e 's/:/ /'`
        __index=$1
        key=$2
        value=$3
        
        if [ -z $key ]; then
          break;
        fi

        if [ "$key" == 'ips' ]; then
            index=${value:0:1}
            line1=${value##$index:} # remove index

            read line2
            read line3

            set -- `echo $line2 | sed -e 's/:/ /' -e 's/:/ /'`
            line2=${3##$index:}

            set -- `echo $line3 | sed -e 's/:/ /' -e 's/:/ /'`
            line3=${3##$index:}

            if [ $label == "public" ]; then
                public_addresses=( "${public_addresses[@]}" "$line1;$line2;$line3" )
            elif [ $label == "private" ]; then
                private_addresses=( "${private_addresses[@]}" "$line1;$line2;$line3" )
            fi
        elif [ "$key" == 'routes' ]; then
            index=${value:0:1}
            line1=${value##$index:} # remove index

            read line2
            read line3

            set -- `echo $line2 | sed -e 's/:/ /' -e 's/:/ /'`
            line2=${3##$index:}

            set -- `echo $line3 | sed -e 's/:/ /' -e 's/:/ /'`
            line3=${3##$index:}

            routes=( "${routes[@]}" "$line1;$line2;$line3" )
        elif [ "$key" == 'dns' ]; then
            read dns2
            dns[0]=${value##*:}
            dns[1]=${dns2##*:}
        else
            eval $key=$value 2> /dev/null
        fi
    done
}

function parse_multi_key_line() {
    line=$1

    IFSBAK=$IFS
    IFS=";"

    for var in ${line[@]}; do
        set -- `echo $var | sed -e 's/:/;/'`
        eval "$1=$2"
    done

    IFS=$IFSBAK
}

if [ $BASH_SOURCE == $0 ]; then 
  get_interface_bullshit
  case `${AGENTSMITHDIR}/detect_os.sh` in 
    ("ubuntu"|"debian") 
        ubuntu_network_config 

        if [ ! $DEBUG ]; then 
          /etc/init.d/networking restart restart> /dev/null
        fi
        ;;
    ("redhat"|"fedora"|"oracle") 
        redhat_network_config 

        if [ ! $DEBUG ]; then 
          /etc/init.d/network restart restart> /dev/null
        fi
        ;;
    ("gentoo") 
        gentoo_network_config 

        if [ ! $DEBUG ]; then 
          /etc/init.d/net.eth0 restart restart> /dev/null
          /etc/init.d/net.eth1 restart restart> /dev/null
        fi
        ;;
    ("arch") 
        arch_network_config 

        if [ ! $DEBUG ]; then 
          /etc/rc.d/network restart restart> /dev/null
        fi
        ;;
  esac
fi
