#!/bin/sh
#
#  Copyright 2010 Rackspace US, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
ROOT="${ROOT:-/}"

check_value_and_maybe_return() {
    distributor="$1"

    if echo "$distributor" | grep -qi Ubuntu
    then
        echo ubuntu
        exit 0
    elif echo "$distributor" | grep -qi Debian
    then
        echo debian
        exit 0
    elif echo "$distributor" | grep -qi CentOS
    then
        echo centos
        exit 0
    elif echo "$distributor" | grep -qi Fedora
    then
        echo fedora
        exit 0
    elif echo "$distributor" | grep -qi RedHat
    then
        echo redhat
        exit 0
    elif echo "$distributor" | grep -qi Gentoo
    then
        echo gentoo
        exit 0
    fi
}

lsbrelease="${ROOT}/usr/bin/lsb_release"
if [ -x "${lsbrelease}" ]
then
    distributor="$(${lsbrelease} -is)"
    check_value_and_maybe_return "$distributor"
fi

if [ -f "$ROOT/etc/lsb-release" ]
then
    distributor="$(grep DISTRIB_ID= "$ROOT/etc/lsb-release")"
    check_value_and_maybe_return "$distributor"
fi

etc_issue="${ROOT}/etc/issue"
if [ -e "${etc_issue}" ]
then
    if grep -qi Ubuntu "${etc_issue}"
    then
        echo "ubuntu"
        exit 0
    fi

    if grep -qi Debian "${etc_issue}"
    then
        echo "debian"
        exit 0
    fi

    if grep -qi Centos "${etc_issue}"
    then
        echo "centos"
        exit 0
    fi

    if grep -qi Fedora "${etc_issue}"
    then
        echo "fedora"
        exit 0
    fi

    if grep -qi "Red Hat" "${etc_issue}"
    then
        echo "redhat"
        exit 0
    fi

    if grep -qi "Arch" "${etc_issue}"
    then
        echo "arch"
        exit 0
    fi
fi

if [ -f "$ROOT/etc/enterprise-release" ]
then
    echo "oracle"
    exit 0
fi

if [ -e "$ROOT/etc/redhat-release" ]
then
    if grep -qi CentOS "$ROOT/etc/redhat-release"
    then
        echo "centos"
        exit 0
    fi

    if grep -qi Fedora "$ROOT/etc/redhat-release"
    then
        echo "fedora"
        exit 0
    fi

    if grep -qi "Red Hat" "$ROOT/etc/redhat-release"
    then
        echo "redhat"
        exit 0
    fi
fi

if [ -f "$ROOT/etc/gentoo-release" ]
then
    echo "gentoo"
    exit 0
fi

if [ -f "$ROOT/etc/debian_version" ] && grep -q 4 "$ROOT/etc/debian_version"
then
    echo "debian"
    exit 0
fi

# Shot in the dark
echo "ubuntu"
exit 0
