#!/bin/bash
#
#  Copyright 2010 Rackspace US, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# SANE DEFAULTS
E_OPTERROR=85
AGENTSMITHDIR="${AGENTSMITHDIR:-/usr/share/agent-smith}"
AGENTSMITHLIB="${AGENTSMITHLIB:-/usr/local/lib/agent-smith/}"
SPOOL_DIR="${SPOOL_DIR:-/var/spool/agent}"
PATH=$PATH:$AGENTSMITHLIB
export TEMP=/tmp

mkdir ${TEMP}/agent-smith/

# The C portion of the agent writes things out from _its_
# perspective. Wonky, but this works
INCOMING="$SPOOL_DIR/outgoing"
OUTGOING="$SPOOL_DIR/incoming"

. $AGENTSMITHDIR/key_exchange.sh
. $AGENTSMITHDIR/reset_password.sh

function usage() {
  echo "Usage: `basename $0` -s SPOOL_DIR [-d AGENTSMITHDIR]"
}

function _function_exists() {
  declare -f $1 > /dev/null; return $?
}

function version() {
  cat $AGENTSMITHDIR/VERSION
}

function dispatch() { 
  outfile=$1

  if (_function_exists $action); then
      response_code=0
      ackmsg=`eval $action $(cat ${TEMP}/agent-smith/$tmpfile)}`
  else
      ackmsg=`cat ${TEMP}/agent-smith/$tmpfile | $AGENTSMITHDIR/$action`
      response_code=$?
  fi 

  rm ${TEMP}/agent-smith/$tmpfile
  echo "{ \"response_code\": $response_code, \"ackmsg\": \"$ackmsg\" }" > $OUTGOING/$outfile
}

function read_file() {
    file=$1
    tmpfile=${file##$INCOMING}

    if [ -e $file ]; then
        IFSBAK=$IFS
        IFS=$'\n'

        for line in `cat $file | json2flat`; do
          if [[ "$line" =~ "name:" ]]; then
            action=${line##name:}
          else
            # Stuff it into a tempfile 
            echo ${line##value:} >> ${TEMP}/agent-smith/$tmpfile
          fi
        done


        unlink $file
    fi
}

function parse_message() {
    case $1 in 
    "kmsactivate")  action="kms_activate.sh";;
    "agentupdate")  action="agent_update.sh";;
    "keyinit")      action="dh_key_init";;
    "password")     action="reset_password.sh";;
    "resetnetwork") action="configure_network.sh";;
    "version")      action="version";;
    "expanddisk");; # DO NOTHING
    esac
}

if [ $# -eq 0 ]    # Script invoked with no command-line args?
then
  usage
  exit $E_OPTERROR
fi  

name=$1
# read the file from $INCOMING
read_file $INCOMING/$name
if [ -n $action ]; then
    parse_message $action
    dispatch $name
fi
