#!/bin/sh
#
#  Copyright 2010 Rackspace US, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

export AGENTSMITHDIR="${AGENTSMITHDIR:-/usr/share/agent-smith}"
export ROOT="${ROOT:-}"
export RHN_PATH="${ROOT}/etc/sysconfig/rhn"

function register_with_rhn() { 
    # Remove the existing redhat network system id
    rm -f $RHN_PATH/systemid

    if [ ! $DEBUG ]; then
        rhnreg_ks --activationkey $activation_key --profile $profile --force
    fi
}

function get_kms_variables() {
    while read line; do
      set -- `echo $line | sed -e 's/:/ /'`
      key=$1
      value=$2

      if [ $key == 'domains' ]; then
          http_proxies="${http_proxies}noSSLServerURL=http://${value}/XMLRPC;"
          https_proxies="${https_proxies}ServerURL=https://${value}/XMLRPC;"
      else 
          eval $key=$value 2> /dev/null
      fi

    done
}

function configure_up2date() {
    backupfile=$(make_backup $RHN_PATH/up2date)

    if [ ! -f "${backupfile}" ]; then
      exit 1
    fi

    # generate up2date configuration from template
    cat > $RHN_PATH/up2date<<UP2DATE
# Automatically generated Red Hat Update Agent config file, do not edit.
# Format: 1.0
versionOverride[comment]=Override the automatically determined system version
versionOverride=

enableProxyAuth[comment]=To use an authenticated proxy or not
enableProxyAuth=0

networkRetries[comment]=Number of attempts to make at network connections before giving up
networkRetries=5

hostedWhitelist[comment]=None
hostedWhitelist=

enableProxy[comment]=Use a HTTP Proxy
enableProxy=0

serverURL[comment]=Remote server URL
$https_proxies 

proxyPassword[comment]=The password to use for an authenticated proxy
proxyPassword=

noSSLServerURL[comment]=None
$http_proxies

proxyUser[comment]=The username for an authenticated proxy
proxyUser=

disallowConfChanges[comment]=Config options that can not be overwritten by a config update action
disallowConfChanges=noReboot;sslCACert;useNoSSLForPackages;noSSLServerURL;serverURL;disallowConfChanges;

sslCACert[comment]=The CA cert used to verify the ssl server
sslCACert=/usr/share/rhn/RHN-ORG-TRUSTED-SSL-CERT

debug[comment]=Whether or not debugging is enabled
debug=0

httpProxy[comment]=HTTP proxy in host:port format, e.g. squid.redhat.com:3128
httpProxy=

systemIdPath[comment]=Location of system id
systemIdPath=/etc/sysconfig/rhn/systemid

noReboot[comment]=Disable the reboot action
noReboot=0

UP2DATE
}
