function arch_network_config() {
    ARCH_CONFIG_FILE="/etc/rc.conf"
    TEMP_ARCH_CONFIG="${TEMP}/${ARCH_CONFIG_FILE}"
    INIT_DIR="${ROOT}/etc/init.d"
    RUNLEVELS_DIR="${ROOT}/etc/runlevels/default/"

    backupfile="$(make_backup ${ROOT}/$ARCH_CONFIG_FILE)"

    if ! [ -f "${backupfile}" ]
    then
        echo "NO backupfile"
        exit 1
    fi

    if [ ! -d ${TEMP_ARCH_CONFIG%%rc.conf} ]; then
        mkdir -p ${TEMP_ARCH_CONFIG%%rc.conf}
    fi

    public_address_count=0
    public_address_iface=''
    interfaces='lo eth0'
    output=''
    gateways="gateway=\"default gw ${gateway}\"\\
    "
    gateway_routes="gateway"

    # CONFIGURE PUBLIC INTERFACES
    for address in ${public_addresses[@]}; do
        parse_multi_key_line $address

        this_interface=""

        if [ $public_address_count -eq 0 ]; then
          this_interface="eth0=\"eth0"
        elif [ $public_address_count -gt 0 ]; then
          interface="eth0x${public_address_count}"
          this_interface="$interface=\"eth0:${interface##eth0x}"
          if [ $enabled -eq 1 ]; then
            interfaces="$interfaces $interface"
          fi
        fi

        this_interface="$this_interface $ip netmask $netmask\"\\
        "
        output="${output}${this_interface}"

        let public_address_count=$public_address_count+1
        unset ip netmask enabled
    done

    # CONFIGURE PRIVATE INTERFACE AND ROUTES
    if [ ${#private_addresses[@]} -eq 1 ]; then
        parse_multi_key_line ${private_addresses[0]}

        this_interface="eth1=\"eth1 $ip netmask $netmask\"\\"
        output=${output}${this_interface}
        interfaces="$interfaces eth1"

        route_count=0
        for route in ${routes[@]}; do
            parse_multi_key_line $route

            this_route="route${route_count}=\"-net $route netmask $netmask gw $gateway\"\\
            "

            gateways=${gateways}${this_route}
            gateway_routes="$gateway_routes route${route_count}"

            let route_count=$route_count+1
            unset route netmask gateway
        done

        unset ip netmask enabled
    fi

    sed "/^eth[0-9].*/{d;};/INTERFACES=(.*)/{c\ 
    ${output}
    a\
    INTERFACES=($interfaces)
    }" $ROOT/$ARCH_CONFIG_FILE > $TEMP_ARCH_CONFIG

    sed "/^gateway=/{d;};/^route[0-9]=/{d;};/ROUTES=/{c\ 
    ${gateways}
    a\ 
    ROUTES=(${gateway_routes})
    }" $ROOT/$ARCH_CONFIG_FILE > $TEMP_ARCH_CONFIG

    mv $TEMP_ARCH_CONFIG $ROOT/$ARCH_CONFIG_FILE
    rmdir ${TEMP_ARCH_CONFIG%%rc.conf}

    if [ ! -L $INIT_DIR/net.eth1 ]; then
        ln -s $INIT_DIR/net.eth0 $INIT_DIR/net.eth1
        ln -s $INIT_DIR/net.eth1 $RUNLEVELS_DIR/net.eth1
    fi
}
