#!/bin/sh
#
#  Copyright 2010 Rackspace US, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

ROOT="${ROOT:-}"
AGENTSMITHDIR="${AGENTSMITHDIR:-/usr/share/agent-smith}"

. "${AGENTSMITHDIR}/common.sh"

hostname="$1"
firstip="$2"

update_etc_hostname() {
	echo ${hostname} > "${ROOT}/etc/hostname"
}

update_etc_hosts() {
	# This is tricky
    # We need to be careful not to scribble over the existing entries
    # in /etc/hosts, but also be thorough enough to actually make it work
    backupfile="$(make_backup "${ROOT}/etc/hosts")"
    if ! [ -f "${backupfile}" ]
    then
        exit 1
    fi
	cat "${backupfile}" | while read ip tail
	do
		if [ "x$ip" = "x$firstip" ]
		then
            # If there's just a single hostname, assume it's the one we
            # need to replace.
			if echo "$tail" | grep -iq '^[a-z][a-z0-9]*$'
			then
                echo "#$ip $tail # Removed by Agent Smith"
                echo "$ip $hostname"
            elif echo "$tail" | grep -q '[ 	]'
			then
				# Ok, more than one word in the line..
                # Without the while loop, a, b, and c are not available
                # afterwards
				echo "$tail" | while read a b c
				do
                    if [ -n "$c" ] && ! echo "$c" | grep -q "^#"
                    then
                        # Three words.. Don't know how to handle that.
                        echo "$ip $tail"
                    elif echo "$a" | grep -q '\.' && echo echo "$b" | grep -q '\.'
                    then
                        # Two words, both with a dot in them.. Don't know how to handle that.
                        echo "$ip $tail"
                    elif ! echo "$a" | grep -q '\.' && ! echo echo "$b" | grep -q '\.'
                    then
                        # Two words, neither with a dot in them.. Don't know how to handle that.
                        echo "$ip $tail"
                    else
                        # If there's not a dot in both, nor are both missing a dot,
                        # exactly one of them must have a dot. Yay.
                        withdot="$(echo $a | grep '\.' ; echo $b | grep '\.')"
                        withoutdot="$(echo $a | grep -v '\.' ; echo $b | grep -v '\.')"
                        if [ "${withdot%%.*}" != "${withoutdot}" ]
                        then
                            # If the host part doesn't match, I don't know what to do.
                            echo "$ip $tail"
                        else
                            domain="${withdot#*.}"
                            echo "#$ip $tail # Removed by Agent Smith"
                            echo "$ip $hostname.$domain $hostname"
                        fi
                    fi
                done
            else
                echo "$ip $tail"
            fi
        else
            echo "$ip $tail"
        fi
    done > "${ROOT}/etc/hosts"
}

update_etc_hostname
update_etc_hosts

exit 0
