/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <json/json.h>
#include "util.h"

#define output(...) fprintf(fp, __VA_ARGS__)

static char *_default_separators = ":";

FILE *fp;

void append_primitive_value(strbuf *str, json_object *val, const char *prefix) {
    strbuf_append_printf(str, "%s", prefix);
    switch (json_object_get_type(val)) {
        case json_type_null:
            break;
        case json_type_boolean:
            strbuf_append_printf(str, "%s", json_object_get_boolean(val) ? "1" : "0");
            break;
        case json_type_double:
            strbuf_append_printf(str, "%.2f", json_object_get_double(val));
            break;
        case json_type_int:
            strbuf_append_printf(str, "%d", json_object_get_int(val));
            break;
            break;
        case json_type_string:
            strbuf_append_printf(str, "%s", json_object_get_string(val));
			break;
        case json_type_object:
        case json_type_array:
		default:
            /* We can't deal with these anyways, so we ignore them. */
			break;
    }
    strbuf_append_printf(str, "\n");
}

strbuf *append_json_value(strbuf *str, struct json_object *val, int level, const char *prefix, char *separator) {

    if (*separator == '\0')
        separator--;

    switch (json_object_get_type(val)) {
        case json_type_null:
        case json_type_boolean:
        case json_type_double:
        case json_type_int:
        case json_type_string:
            append_primitive_value(str, val, prefix);
            break;
        case json_type_object:
            {
                struct lh_entry *entry;

                for(entry = json_object_get_object(val)->head; entry; entry = entry->next) {
                    strbuf *keybuf = strbuf_init();
                    char *key;

                    strbuf_append_printf(keybuf, "%s%s%c", prefix, (char*)entry->k, *separator);
                    append_json_value(str, (struct json_object*)entry->v, level+1, key = strbuf_free(keybuf, 0), separator + 1);

                    free(key);
                }
            }
            break;
        case json_type_array:
            {
                int i;
                array_list *list;

                list = json_object_get_array(val);

                for (i=0; i<list->length; i++) {
                    strbuf *keybuf = strbuf_init();
                    char *key;

                    strbuf_append_printf(keybuf, "%s%d%c", prefix, i, *separator);
                    append_json_value(str, list->array[i], level+1, key = strbuf_free(keybuf, 0), separator + 1);

					free(key);
                }
            }
        break;
    }
    return str;
}

char *reserialize_json(const char *json_string, char *separators)
{
    struct json_object *jobj;
    strbuf *str = strbuf_init();

    if (separators == NULL || *separators == '\0')
        separators = _default_separators;

    jobj = json_tokener_parse(json_string);

    if ((signed long) jobj > 0) {
        append_json_value(str, jobj, 0, "", separators);
    }
    return strbuf_free(str, 0);
}
