/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>
#include "agent.h"

extern int log_to_stdout;

#define PIDFILE "/var/run/agent-smith.pid"

void print_usage(int long_usage, char *cmd) {
    fprintf(stderr, "Usage: %s [-f] [-h] dispatcher\n", cmd);
    if (long_usage) {
        fprintf(stderr, "\n");
        fprintf(stderr, "-f   Run in the foreground\n");
        fprintf(stderr, "-h   This help screen\n");
    }
}

int main(int argc, char *argv[])
{
    int daemonise = 1;
    int opt;
    char *pidfile;
    FILE *fp;

    while ((opt = getopt(argc, argv, "fh")) != -1) {
        switch (opt) {
            case 'f':
                daemonise = 0;
                log_to_stdout = 1;
                break;
            case 'h':
                print_usage(1, argv[0]);
                exit(EXIT_SUCCESS);
                break;
            default:
                print_usage(0, argv[0]);
                exit(EXIT_FAILURE);
        }
    }

    if (optind >= argc) {
        print_usage(0, argv[0]);
        exit(EXIT_FAILURE);
    }

    pidfile = getenv("AGENT_SMITH_PIDFILE");
    if (!pidfile)
        pidfile = PIDFILE;

    if ((fp = fopen(pidfile, "w")) == NULL) {
        perror("Opening pid file");
        return 1;
    }

    if (daemonise) {
        if (daemon(0, 0) != 0) {
            perror("Error while daemonizing");
            return 1;
        }
    }

    fprintf(fp, "%d\n", getpid());
    fclose(fp);

    if (agent_init(argv[optind]) < 0) {
        syslog(LOG_DAEMON | LOG_CRIT, "Agent initialisation failed.");
        return -1;
    }

    while (!agent_loop());

    return 0;
}
