/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#ifndef __SRC_SLIST_H__
#define __SRC_SLIST_H__

typedef struct _SList SList;
typedef struct _SListElem SListElem;

typedef void (*slist_destroy_cb)(void *data);

/**
 * Create a new list for storing arbitrary data
 *
 * @param destroy_cb Callback to call when deiniting (Can be NULL)
 * @retval A new list that is init'd as empty
 */
SList *slist_init(slist_destroy_cb destroy_cb);

/**
 * Destroy a previously created list
 *
 * @param slist The list to destroy
 */
void slist_deinit(SList *slist);

/**
 * Append a new element to a previously created slist
 *
 * @param list The list to append to
 * @param data Arbitrary data pointer
 * @retval The newly created element
 */
SListElem *slist_append(SList *list, void *data);

/**
 * Return the first element of a previously created list
 *
 * @param list The list
 * @retval The first element or NULL if none
 */
SListElem *slist_first(SList *list);

/**
 * Return the last element of a previously created list
 *
 * @param list The list
 * @retval The last element or NULL if none
 */
SListElem *slist_last(SList *list);

/**
 * Return the next element in the list after the passed element
 *
 * @param list The list
 * @param elem The current element
 * @retval The element after the current element or NULL if none left
 */
SListElem *slist_next(SList *list, SListElem *elem);

/**
 * Return the previous element in the list after the passed element
 *
 * @param list The list
 * @param elem The current element
 * @retval The element before the current element or NULL if none
 */
SListElem *slist_prev(SList *list, SListElem *elem);

/**
 * Return the arbitrary data pointer for an element
 *
 * @param elem The element you want the data for
 * @retval The arbitrary data pointer
 */
void *slist_data(SListElem *elem);


#endif /* __SRC_SLIST_H__ */
