/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#ifndef SPOOL_H
#define SPOOL_H

typedef void (*spool_callback)(const char *path);

int spool_init(void);
char *spool_incoming_dir(void);
char *spool_outgoing_dir(void);
int agent_setup_signals(void);

/**
 * Store an incoming message in the spool
 *
 * @param msgid unique identifier for the message
 * @param buf pointer to the message contents
 * @param buflen length of message contents
 * @return -1 on error, 0 otherwise
 */
int spool_store(const char *msgid, const void *buf, unsigned int buflen);

/**
 * Register a watch on the incoming spool.
 *
 * When a new file lands in the spool, cb gets called with the full path
 * of the new file as its only argument.
 *
 * @param cb Function to call when a new file is created in the spool
 * @return 1 on success, 0 otherwise
 */
int spool_watch(spool_callback cb);

/**
 * Process pending watch events
 *
 * Fires any callbacks that are pending, then returns. If no events are
 * pending, returns immediately.
 */
void spool_process_pending_events(void);

/* void *spool_retrieve(char **msgid, unsigned int *buflen); */
int spool_delete(const char *msgid);

#endif /* SPOOL_H */
