/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <check.h>
#include <unistd.h>
#include "../src/agent.h"
#include "../src/xen.h"
#include "../src/spool.h"

char spooldir[255];
char *msgfile;

void test_spool_to_xen_setup(void) {
    strcpy(spooldir, "myspoolXXXXXX");
    fail_unless(mkdtemp((char *)spooldir) != NULL);
    setenv("AGENT_MSG_SPOOL", spooldir, 1);
}

void test_spool_to_xen_teardown(void) {
    if (msgfile)
        unlink(msgfile);
}

int    spool_to_xen_cb_called = 0;

int spool_to_xen_cb(const char *path, const char *buf, const size_t buflen, const char *token) {
    spool_to_xen_cb_called = 1;
    return 0;
}

START_TEST(test_xen_to_spool)
{
    char xs_in_msg_path[255], msg_file_path[255], *msgid = "testmsg", *outgoing_spool;
    struct stat sb;

    fail_unless(agent_init("/bin/true") == 0, "agent_init() failed");

    snprintf(xs_in_msg_path, 255, XS_MSG_INCOMING_BASE_PATH "/%s", msgid);
    xen_write_path(xs_in_msg_path, "Foo", 3);

    outgoing_spool = spool_outgoing_dir();
    snprintf(msg_file_path, 255, "%s/%s", outgoing_spool, msgid);

    agent_loop();

    fail_unless(stat(msg_file_path, &sb) == 0, "File did not get created.");
}
END_TEST

START_TEST(test_spool_to_xen)
{
    FILE *fp;
    char watch_path[255], *msgid;
    char *incoming_spool;

    fail_unless(agent_init("/bin/true") == 0, "agent_init() failed");

    incoming_spool = spool_incoming_dir();
    msgfile = tempnam(incoming_spool, NULL);
    msgid = strrchr(msgfile, '/');

    snprintf(watch_path, 255, XS_MSG_OUTGOING_BASE_PATH "/%s", msgid);
    xen_register_watch(watch_path, spool_to_xen_cb, "sometoken");

    fp = fopen(msgfile, "w");
    fwrite("Hello", 1, 5, fp);
    fclose(fp);

    fail_unless(!spool_to_xen_cb_called, "Callback already fired?");
    agent_loop();
    fail_unless(spool_to_xen_cb_called, "Callback didn't fire");
}
END_TEST

START_TEST(test_agent_signals)
{
    fail_unless(agent_init("/bin/true") == 0, "agent_init() failed");

    fail_unless(kill(getpid(), SIGCHLD) == 0);
    fail_unless(kill(getpid(), SIGCHLD) == 0);
    fail_unless(kill(getpid(), SIGPIPE) == 0);
    fail_unless(kill(getpid(), SIGTERM) == 0);
    /* agent_loop() should return != 0 for receiving SIGTERM */
    fail_unless(agent_loop() != 0);
}
END_TEST

Suite *agent_suite(void)
{
    Suite *s = suite_create("Agent");

    TCase *tc_agent = tcase_create("Agent");
    tcase_add_checked_fixture(tc_agent, test_spool_to_xen_setup, test_spool_to_xen_teardown);
    tcase_add_test(tc_agent, test_spool_to_xen);
    tcase_add_test(tc_agent, test_xen_to_spool);
    tcase_add_test(tc_agent, test_agent_signals);
    suite_add_tcase(s, tc_agent);

    return s;
}

int main(void)
{
    int number_failed;

    Suite *s = agent_suite();
    SRunner *sr = srunner_create(s);
    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
