/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <stdlib.h>
#include <stdio.h>
#include <check.h>
#include "../src/json.h"

#define json_test(in, out) fail_unless(strcmp(tmp = reserialize_json(in), out) == 0, "Got: \"%s\"", tmp)

START_TEST(test_reserialize_json)
{
	char *tmp;
    json_test("'foo'", "foo\n");
    json_test("{ 'foo' : 'bar' }", "foo:bar\n");
    json_test("{ 'foo' : [ 'bar', 'baz'] }", "foo:0#bar\nfoo:1#baz\n");
    json_test("{ 'foo' : { 'bar' : 'baz', 'key': 'value' } }", "foo:bar#baz\nfoo:key#value\n");
    json_test("{ 'foo' : { 'bar' : { 'baz' : { 'wibble' : 'wobble' } } } }", "foo:bar#baz%wibble^wobble\n");
    json_test("{\"foo\": [[[\"bar\"]]]}", "foo:0#0%0^bar\n");
    json_test("{ 'foo' : { 'bar' : 'baz', 'key' : [ 'value', { 'foo' : 'bar' } ] } }", "foo:bar#baz\nfoo:key#0%value\nfoo:key#1%foo^bar\n");
}
END_TEST

Suite *json_suite(void)
{
    Suite *s = suite_create("JSON");

    TCase *tc_json = tcase_create("json");
    tcase_add_test(tc_json, test_reserialize_json);
    suite_add_tcase(s, tc_json);

    return s;
}

int main(void)
{
    int number_failed;

    Suite *s = json_suite();
    SRunner *sr = srunner_create(s);
    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
