/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <stdlib.h>
#include <check.h>
#include <assert.h>
#include "../src/slist.h"

START_TEST(test_slist_append)
{
	char *testdata[] = { "foo", "bar", "baz" };
	SList *list = NULL;
	SListElem *elem;
	SListElem *elem1;
	SListElem *elem2;
	SListElem *elem2_2;
	SListElem *elem3;

        fail_unless((list = slist_init(NULL)) != NULL, "initing failed");

	fail_unless(slist_append(list, (void *) testdata[0]) != NULL, "appending failed");

        fail_unless((elem = slist_first(list)) != NULL, "couldn't get first elem");

	fail_unless(slist_last(list) == elem, "last element is not same as first element");

	fail_unless(slist_data(elem) == testdata[0], "element's data attribute not correctly set");
	fail_unless(slist_next(list, elem) == NULL, "next element is not NULL");
	fail_unless(slist_prev(list, elem) == NULL, "prev element is not NULL");

	fail_unless((elem2 = slist_append(list, (void *)testdata[1])) != NULL);

        fail_unless((elem1 = slist_first(list)) != NULL, "couldn't get first elem");

        fail_unless((elem1 == elem), "first element doesn't match original first element");

	fail_unless(slist_data(elem) == testdata[0], "first element's data attribute not correctly set");

	fail_unless(slist_next(list, elem) == elem2, "1st element's next ptr is not 2nd element");
	fail_unless(slist_prev(list, elem) == NULL, "1st element's prev ptr is not NULL");
	fail_unless(slist_prev(list, elem2) == elem, "2nd element's prev ptr is not 1st element");
	fail_unless(slist_next(list, elem2) == NULL, "2nd element's next ptr is not NULL");
	fail_unless(slist_last(list) == elem2, "last element is not same as 2nd element");

	fail_unless(slist_data(elem2) == testdata[1], "2nd element's data attribute not correctly set");

	fail_unless((elem3 = slist_append(list, (void *)testdata[2])) != NULL);

        fail_unless((elem1 = slist_first(list)) != NULL, "couldn't get first elem");

        fail_unless((elem1 == elem), "first element doesn't match original first element");

        fail_unless((elem2_2 = slist_next(list, elem1)) != NULL, "couldn't get 2nd elem");

        fail_unless((elem2_2 == elem2), "2nd element doesn't match original 2nd element");

	fail_unless(slist_data(elem) == testdata[0], "first element's data attribute not correctly set");

	fail_unless(slist_next(list, elem) == elem2, "1st element's next ptr is not 2nd element");
	fail_unless(slist_prev(list, elem) == NULL, "1st element's prev ptr is not NULL");
	fail_unless(slist_prev(list, elem2) == elem, "2nd element's prev ptr is not 1st element");
	fail_unless(slist_next(list, elem2) == elem3, "2nd element's next ptr is not third element");
	fail_unless(slist_prev(list, elem3) == elem2, "3rd element's prev ptr is not 2nd element");
	fail_unless(slist_next(list, elem3) == NULL, "3rd element's next ptr is not NULL");
	fail_unless(slist_last(list) == elem3, "last element is not same as 3rd element");

	fail_unless(slist_data(elem3) == testdata[2], "3rd element's data attribute not correctly set");

        slist_deinit(list);
}
END_TEST

int destroy_cb_iter;
char *destroy_cb_testdata[] = { "foo", "bar", "baz" };

void _slist_destroy_cb(void *arg)
{
    switch(++destroy_cb_iter)
    {
        case 1:
            assert(arg == destroy_cb_testdata[0]);
            break;
        case 2:
            assert(arg == destroy_cb_testdata[1]);
            break;
        case 3:
            assert(arg == destroy_cb_testdata[2]);
            break;
        case 4:
            break;
    }
}

START_TEST(test_slist_destroy_cb)
{
	SList *list = NULL;

        destroy_cb_iter = 0;

        fail_unless((list = slist_init(NULL)) != NULL, "initing failed");

	fail_unless(slist_append(list, (void *) destroy_cb_testdata[0]) != NULL, "appending failed");
	fail_unless(slist_append(list, (void *) destroy_cb_testdata[1]) != NULL, "appending failed");
	fail_unless(slist_append(list, (void *) destroy_cb_testdata[2]) != NULL, "appending failed");

        slist_deinit(list);

        fail_unless(destroy_cb_iter == 0);

        fail_unless((list = slist_init(_slist_destroy_cb)) != NULL, "initing failed");

	fail_unless(slist_append(list, (void *) destroy_cb_testdata[0]) != NULL, "appending failed");
	fail_unless(slist_append(list, (void *) destroy_cb_testdata[1]) != NULL, "appending failed");
	fail_unless(slist_append(list, (void *) destroy_cb_testdata[2]) != NULL, "appending failed");

        slist_deinit(list);

        fail_unless(destroy_cb_iter == 3);
}
END_TEST

Suite *slist_suite(void)
{
    Suite *s = suite_create("SList");

    TCase *tc_slist = tcase_create("slist");
    tcase_add_test(tc_slist, test_slist_append);
    tcase_add_test(tc_slist, test_slist_destroy_cb);
    suite_add_tcase(s, tc_slist);

    return s;
}

int main(void)
{
    int number_failed;

    Suite *s = slist_suite();
    SRunner *sr = srunner_create(s);
    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
