/*
 *  Copyright 2010 Rackspace US, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
#include <check.h>
#include <stdio.h>
#include <stdlib.h>
#include "../src/util.h"

START_TEST(test_util_join_paths)
{
    fail_unless(strcmp("foo/bar", util_join_paths("foo", "bar")) == 0);
}
END_TEST

START_TEST(test_strbuf_sprintf)
{
    strbuf *s = strbuf_init();
    strbuf_append_printf(s, "%s/%d/%.2f", "foo", 12345, 0.234);
    fail_unless(strcmp("foo/12345/0.23", s->str) == 0);
    strbuf_append_printf(s, "%d%s%d", 4321, "bar", 5678);
    fail_unless(strcmp("foo/12345/0.234321bar5678", s->str) == 0);
}
END_TEST


Suite *util_suite(void)
{
    Suite *s = suite_create("Util");

    TCase *tc_util = tcase_create("util");
    tcase_add_test(tc_util, test_util_join_paths);
    tcase_add_test(tc_util, test_strbuf_sprintf);
    suite_add_tcase(s, tc_util);

    return s;
}

int main(void)
{
    int number_failed;

    Suite *s = util_suite();
    SRunner *sr = srunner_create(s);
    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
